#!/bin/ksh
#
# Netcool/Proviso
# Copyright (c) 2005 Micromuse, Inc.  All rights reserved.
#
# NAME
#    timezone_patch.ksh
#
# DESCRIPTION
#    This script implements the new TZ functionality
#
# HISTORY   (MM-DD-YYYY)
#    tjoh    12-15-2006  Script creation
#

# Variables definition

# ----- Type declarations -----


typeset    PV_USER
typeset    PV_PASSWD
typeset    PV_PASSWD_CHK
typeset    PV_MENU_PASS=''
typeset    STARS='********'

typeset -x ORACLE_SID
typeset -x ORACLE_HOME

typeset -r CONFIG_FILE_NAME=database.conf
typeset    CONFIG_FILE=$PVMHOME/conf/$CONFIG_FILE_NAME

typeset -r REVISION="$Revision:   1.6  $"

typeset    SPOOL_FILE=/tmp/tz_patch_temp_$$.tmp

alias EchoOn='tput sgr0'
alias EchoOff='tput invis'

# Procedures

init()
{
   _uname=`uname -s`

   case $_uname in

      'SunOS')
         WHOAMI=`/usr/ucb/whoami`
         UNAME=`uname -rsv`
         HOSTNAME=`hostname`
         ORA_PATH='$ORACLE_HOME/bin:/usr/ccs/bin'
         SHLIB_PATH_VAR='LD_LIBRARY_PATH'
         LIBEXT=so         
         ;;

      'AIX')
         WHOAMI=`/usr/bin/whoami`
         UNAME=`uname -rsv`
         HOSTNAME=`hostname`
         ORA_PATH='$ORACLE_HOME/bin:/usr/ccs/bin'
         SHLIB_PATH_VAR='LIBPATH'
         LIBEXT=so
         ;;
   esac
}

menu_header()
{
   tput clear
   echo "--------------------------------------------------"
   echo "Netcool/Proviso Datamart"
   echo "Date: $(date)"
   echo "Script name: timezone_patch.ksh"
   echo "--------------------------------------------------"
   echo " "
   echo "  Database user................. : [ $PV_USER ]"
   echo "  Database user password........ : [ $PV_MENU_PASS ]"
   echo " "
}

menu_body()
{
   echo "Menu :"

   echo " "
   echo "   1. Input password for $PV_USER."
   echo "   2. Run timezone patch."
   echo "   0. Exit"
   echo " "
}

check_proviso_home()
{
   if [ "$PVMHOME" = '' ]
   then
      echo ""
      echo "PVMHOME not found."
      echo "Did you source the dataMart.env?"
      pause
      return 1
   fi
}

check_oracle_home()
{
   if [ "$ORACLE_HOME" = '' ]
   then 
      echo ""
      echo "ORACLE_HOME not found."
      echo "Did you source the dataMart.env?"
      pause
      return 1
   fi
}

get_oracle_username()
{
   if [ ! -f "$CONFIG_FILE" ]
   then
      echo "$CONFIG_FILE not found."
      echo "Abort"
      CleanExit 1
   else
      PV_USER=$( cat $CONFIG_FILE \
                 | grep '^\*.*user' \
                 | cut -f2 -d: \
                 | sed -e 's/[     ]*//' \
                 | tr '[:lower:]' '[:upper:]' )
      PV_USER=$PV_USER"_ADMIN"
   fi
}

check_parms()
{
   if [ "$PV_MENU_PASS" = '' ]
   then
     echo ""
     echo "You must enter a password"
     pause
     return 1
   fi
   validate_password
   if [ $? -ne 0 ]
   then
     return 1
   fi
}

validate_password()
{
   echo "WhenEver SQLError Exit 1
         Connect ${PV_USER}/${PV_PASSWD}@$ORACLE_SID
         Exit 0
   " | $ORACLE_HOME/bin/sqlplus -S /NOLOG >/dev/null
 
   if [ $? -eq 0 ]
   then
     break
   else
     echo " "
     echo "Problem validating password"
     echo "Please make sure you are entering the correct password"
     pause
     return 1
   fi
}

check_db_install()
{
   rm -fR ${SPOOL_FILE}
   echo "WhenEver SQLError Exit 1
         Connect ${PV_USER}/${PV_PASSWD}@$ORACLE_SID
         Set Feedback Off
         Set Pagesize 0
         Set Linesize 80
         Spool ${SPOOL_FILE}
         Select 'TZ_DB_DONE'
           From REG\$DATABASE
          Where Str_Path = 'DATABASE.GENERAL.TZ.PATCH_DATE'
             Or (     Str_Path = 'DATABASE.GENERAL.VLN'
                  And Str_Value = 'DB50011406000' );
         Spool Off
         Exit 0
   " | $ORACLE_HOME/bin/sqlplus -S /NOLOG >/dev/null
 
   PATCH_COUNT=`grep -c TZ_DB_DONE ${SPOOL_FILE}`
#
#  If the PATCH_COUNT is 1 (the base patch is installed) then return a successful (0) return code
#  If the PATCH_COUNT is 0 (the base patch is not installed) the return an unsuccessful (1) return code
#
   if [ ${PATCH_COUNT} -eq 0 ] 
   then
     return 1
   else
     return 0
   fi 
}

get_current_level()
{
   current_level=`$ORACLE_HOME/bin/sqlplus -s ${PV_USER}/${PV_PASSWD} << EOF
                  WHENEVER SQLERROR EXIT FAILURE ROLLBACK
                  SET HEADING OFF
                  SET FEEDBACK OFF
                  SET PAGES 0
                  Select Distinct ( Substr ( Str_Path, 37, 1 ) )
                    From REG_DATABASE
                   Where Str_Path Like 'DATABASE.GENERAL.PATCHES.DB_PATCH_TZ%'
                     And Str_Value = (Select Distinct ( Max ( Str_Value ) )
                                        From REG_DATABASE
                                       Where Str_Path Like 'DATABASE.GENERAL.PATCHES.DB_PATCH_TZ%');
                  exit success
                  EOF`

  len="`expr "$current_level" : '[0-9]*'`"
  if [ $len = 0 ]
  then 
      current_level=0
  fi 

}

get_password()
{
   echo " "
   echo "==> Enter password for $PV_USER : \c"
   EchoOff
   read PV_PASSWD
   EchoOn
   echo " "
   echo "==> Re-enter password : \c"
   EchoOff
   read PV_PASSWD_CHK
   EchoOn
   if [ "$PV_PASSWD" != "$PV_PASSWD_CHK" ]
   then
     echo " "
     echo "They don't match !"
     PV_PASSWD=''
     pause
   else
     validate_password
   fi
}

update_tables()
{
   echo " "
   echo "Installing database support..."

   rm -fR ${SPOOL_FILE}
   echo "WhenEver SQLError Exit 1
         Connect ${PV_USER}/${PV_PASSWD}@$ORACLE_SID
         Set Feedback Off
         Set Pagesize 0
         Set Linesize 80
         Spool ${SPOOL_FILE}
         @./sql/init_tz_desc.sql
         @./sql/init_tz_dst_offset.sql
         Spool Off
         Exit 0
   " | $ORACLE_HOME/bin/sqlplus -S /NOLOG >/dev/null

   echo " "
   echo "First step of install complete..."

   if [ $? -ne 0 ]
   then
     return 1
   else
     return 0
   fi

#   more ${SPOOL_FILE}
#   pause
}

run_patch()
{
   while [ 1 ]
   do
      menu_header
      menu_body
 
      echo "Choice : \c"
      read _choice

      case $_choice in

         1) # Modify database user password
            get_password
            if [ "$PV_PASSWD" != '' ] 
            then
              PV_MENU_PASS=$STARS
            fi
            ;;

         2) # Run patch install
            check_parms
            if [ $? = 0 ]
            then
              check_db_install
              if [ $? = 0 ]
              then
                get_current_level
                current_level="`expr ${current_level} + 1`"
                apply_level=$current_level
                update_tables
                if [ $? = 0 ]
                then
                   echo "Updating Aggsets...."
                   while [ $apply_level -le $patch_level ]
                   do
                     $ORACLE_HOME/bin/sqlplus -s $PV_USER/$PV_PASSWD@$ORACLE_SID @sql/apply_$apply_level.sql $apply_level
                     apply_level="`expr $apply_level + 1`"
                   done
                    echo "Patches applied successfully "
                    pause
                else 
                   echo " "
                   echo "ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR"
                   echo "********* Failed to update timezone *********"
                   echo "********          tables             ********"
                   echo "*** Call IBM support for more information ***"
                   echo " "
                fi
              else
                 echo " "
                 echo "ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR"
                 echo "******** Minimum Patch level not met ********"
                 echo "******** You must install the first  ********"
                 echo "******** timezone patch before you   ********"
                 echo "******** install these patches       ********"
                 echo "*** Call IBM support for more information ***"
                 echo " "
              fi
            else
               echo " "
               echo "ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR"
               echo "********        Password Error        ********"
               echo " "
               pause
            fi
            echo " "
            ;;

         0) 
            echo " "
            break
            ;;
      esac
   done
}
pause()
{
   echo "\nPress Enter to continue... \c"
   read _option
}

# =========================================================
#                   M    A    I    N
# =========================================================

init
#
#  Set the patch_level for this patch
#
#  ***** This should be set for every patch release *****
#
patch_level=1
#
#  Make the current level the same as the patch level
#  so by default, nothing is installed if there is a problem. 
#
current_level=$patch_level

check_proviso_home
if [ $? -ne 0 ]
then
  exit 1
fi
check_oracle_home
if [ $? -ne 0 ]
then
  exit 1
fi

get_oracle_username

menu_header
run_patch

rm -fR ${SPOOL_FILE}

exit 0 
